@@ -6,7 +6,14 @@ from typing import List
 from urllib.parse import urljoin
 
 import requests
-from pygrocy.utils import parse_date, parse_float, parse_int, localize_datetime
+
+from pygrocy.utils import (
+    localize_datetime,
+    parse_bool_int,
+    parse_date,
+    parse_float,
+    parse_int,
+)
 
 DEFAULT_PORT_NUMBER = 9192
 
@@ -172,12 +179,6 @@ class ProductData(object):
             parsed_json.get("default_best_before_days", None)
         )
 
-        barcodes_raw = parsed_json.get("barcode", "")
-        if barcodes_raw is None:
-            self._barcodes = None
-        else:
-            self._barcodes = barcodes_raw.split(",")
-
     @property
     def id(self) -> int:
         return self._id
@@ -190,10 +191,6 @@ class ProductData(object):
     def name(self) -> str:
         return self._name
 
-    @property
-    def barcodes(self) -> List[str]:
-        return self._barcodes
-
 
 class ChoreData(object):
     def __init__(self, parsed_json):
@@ -203,12 +200,12 @@ class ChoreData(object):
         self.period_type = parsed_json.get("period_type")
         self.period_config = parsed_json.get("period_config")
         self.period_days = parse_int(parsed_json.get("period_days"))
-        self.track_date_only = parsed_json.get("track_date_only")
-        self.rollover = parsed_json.get("rollover")
+        self.track_date_only = parse_bool_int(parsed_json.get("track_date_only"))
+        self.rollover = parse_bool_int(parsed_json.get("rollover"))
         self.assignment_type = parsed_json.get("assignment_type")
         self.assignment_config = parsed_json.get("assignment_config")
         self.next_execution_assigned_to_user_id = parse_int(
-            "next_execution_assigned_to_user_id"
+            parsed_json.get("next_execution_assigned_to_user_id")
         )
         self.userfields = parsed_json.get("userfields")
 
@@ -321,22 +318,41 @@ class MissingProductResponse(object):
 
 class CurrentVolatilStockResponse(object):
     def __init__(self, parsed_json):
-        self._expiring_products = [
-            CurrentStockResponse(product)
-            for product in parsed_json.get("expiring_products")
-        ]
-        self._expired_products = [
-            CurrentStockResponse(product)
-            for product in parsed_json.get("expired_products")
-        ]
-        self._missing_products = [
-            MissingProductResponse(product)
-            for product in parsed_json.get("missing_products")
-        ]
+        self._due_products = []
+        if "due_products" in parsed_json:
+            self._due_products = [
+                CurrentStockResponse(product)
+                for product in parsed_json.get("due_products")
+            ]
+
+        self._overdue_products = []
+        if "overdue_products" in parsed_json:
+            self._overdue_products = [
+                CurrentStockResponse(product)
+                for product in parsed_json.get("overdue_products")
+            ]
+
+        self._expired_products = []
+        if "expired_products" in parsed_json:
+            self._expired_products = [
+                CurrentStockResponse(product)
+                for product in parsed_json.get("expired_products")
+            ]
+
+        self._missing_products = []
+        if "missing_products" in parsed_json:
+            self._missing_products = [
+                MissingProductResponse(product)
+                for product in parsed_json.get("missing_products")
+            ]
 
     @property
-    def expiring_products(self) -> List[CurrentStockResponse]:
-        return self._expiring_products
+    def due_products(self) -> List[CurrentStockResponse]:
+        return self._due_products
+
+    @property
+    def overdue_products(self) -> List[CurrentStockResponse]:
+        return self._overdue_products
 
     @property
     def expired_products(self) -> List[CurrentStockResponse]:
@@ -347,6 +363,15 @@ class CurrentVolatilStockResponse(object):
         return self._missing_products
 
 
+class ProductBarcode(object):
+    def __init__(self, parsed_json):
+        self._barcode = parsed_json.get("barcode")
+
+    @property
+    def barcode(self) -> str:
+        return self._barcode
+
+
 class ProductDetailsResponse(object):
     def __init__(self, parsed_json):
         self._last_purchased = parse_date(parsed_json.get("last_purchased"))
@@ -360,13 +385,22 @@ class ProductDetailsResponse(object):
 
         self._product = ProductData(parsed_json.get("product"))
 
-        self._quantity_unit_purchase = QuantityUnitData(
-            parsed_json.get("quantity_unit_purchase")
-        )
         self._quantity_unit_stock = QuantityUnitData(
             parsed_json.get("quantity_unit_stock")
         )
 
+        self._parse_location(parsed_json)
+
+        self._parse_barcodes(parsed_json)
+
+    def _parse_barcodes(self, parsed_json):
+        barcodes_raw = parsed_json.get("product_barcodes", "")
+        if barcodes_raw is None:
+            self._barcodes = None
+        else:
+            self._barcodes = [ProductBarcode(barcode) for barcode in barcodes_raw]
+
+    def _parse_location(self, parsed_json):
         raw_location = parsed_json.get("location")
         if raw_location is None:
             self._location = None
@@ -397,6 +431,10 @@ class ProductDetailsResponse(object):
     def last_price(self) -> float:
         return self._last_price
 
+    @property
+    def barcodes(self) -> List[ProductBarcode]:
+        return self._barcodes
+
     @property
     def product(self) -> ProductData:
         return self._product
@@ -467,6 +505,36 @@ class TaskResponse(object):
         self.userfields = parsed_json.get("userfields")
 
 
+class CurrentBatteryResponse(object):
+    def __init__(self, parsed_json):
+        self.id = parse_int(parsed_json.get("battery_id"))
+        self.last_tracked_time = parse_date(parsed_json.get("last_tracked_time"))
+        self.next_estimated_charge_time = parse_date(
+            parsed_json.get("'next_estimated_charge_time")
+        )
+
+
+class BatteryData(object):
+    def __init__(self, parsed_json):
+        self.id = parse_int(parsed_json.get("id"))
+        self.name = parsed_json.get("name")
+        self.description = parsed_json.get("description")
+        self.used_in = parsed_json.get("used_in")
+        self.charge_interval_days = parse_int(parsed_json.get("charge_interval_days"))
+        self.created_timestamp = parse_date(parsed_json.get("row_created_timestamp"))
+        self.userfields = parsed_json.get("userfields")
+
+
+class BatteryDetailsResponse(object):
+    def __init__(self, parsed_json):
+        self.battery = BatteryData(parsed_json.get("battery"))
+        self.charge_cycles_count = parse_int(parsed_json.get("charge_cycles_count"))
+        self.last_charged = parse_date(parsed_json.get("last_charged"))
+        self.next_estimated_charge_time = parse_date(
+            parsed_json.get("'next_estimated_charge_time")
+        )
+
+
 class GrocyApiClient(object):
     def __init__(
         self, base_url, api_key, port: int = DEFAULT_PORT_NUMBER, verify_ssl=True
@@ -671,3 +739,19 @@ class GrocyApiClient(object):
 
     def add_generic(self, entity_type: str, data: object):
         self._do_post_request(f"objects/{entity_type}", data)
+
+    def get_batteries(self) -> List[CurrentBatteryResponse]:
+        parsed_json = self._do_get_request(f"batteries")
+        if parsed_json:
+            return [CurrentBatteryResponse(data) for data in parsed_json]
+
+    def get_battery(self, battery_id: int) -> BatteryDetailsResponse:
+        parsed_json = self._do_get_request(f"batteries/{battery_id}")
+        if parsed_json:
+            return BatteryDetailsResponse(parsed_json)
+
+    def charge_battery(self, battery_id: int, tracked_time: datetime = datetime.now()):
+        localized_tracked_time = localize_datetime(tracked_time)
+        data = {"tracked_time": localized_tracked_time.isoformat()}
+
+        return self._do_post_request(f"batteries/{battery_id}/charge", data)
